import os
import uuid

from django.db import models
from django.conf import settings
from django.core.validators import MinValueValidator
from PIL import Image
from django.dispatch import receiver

class Photo(models.Model):
    image = models.ImageField()
    caption = models.CharField(max_length=128, blank=True)
    uploader = models.ForeignKey(settings.AUTH_USER_MODEL,
                                 on_delete=models.CASCADE)
    date_created = models.DateTimeField(auto_now_add=True)

    IMAGE_MAX_SIZE = (800, 800)

    def resize_image(self):
        image = Image.open(self.image)
        image.thumbnail(self.IMAGE_MAX_SIZE)
        image.save(self.image.path)
        image.close()

    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)
        self.resize_image()


class Blog(models.Model):
    photo = models.ForeignKey(Photo, null=True,
                              on_delete=models.SET_NULL,
                              blank=True)
    title = models.CharField(max_length=128)
    content = models.CharField(max_length=5000)
    word_count = models.IntegerField(validators=[MinValueValidator(0)],
                                      default=0)
    author = models.ForeignKey(settings.AUTH_USER_MODEL,
                               on_delete=models.CASCADE)
    date_created = models.DateTimeField(auto_now_add=True)
    starred = models.BooleanField(default=False)

    def _word_count(self):
        words = self.content.split(" ")
        return len(words)

    def save(self, *args, **kwargs):
        self.word_count = self._word_count()
        super().save(*args, **kwargs)

# These two auto-delete files from filesystem when they are unneeded:
@receiver(models.signals.post_delete, sender=Photo)
def auto_delete_file_on_delete(sender, instance, **kwargs):
    """
    Deletes file from filesystem
    when corresponding `MediaFile` object is deleted.
    """
    if instance.image:
        if os.path.isfile(instance.image.path):
            os.remove(instance.image.path)

@receiver(models.signals.pre_save, sender=Photo)
def auto_delete_file_on_change(sender, instance, **kwargs):
    """
    Deletes old file from filesystem
    when corresponding `MediaFile` object is updated
    with new file.
    """
    if not instance.pk:
        return False

    try:
        old_file = Photo.objects.get(pk=instance.pk).image
    except Photo.DoesNotExist:
        return False

    new_file = instance.image
    if not old_file == new_file:
        if old_file.name and os.path.isfile(old_file.path):
            # TODO: find why sometimes the file is used, not closed and trigger PermissionError
            # seems due to Photo save() overide since the problem does not occur when it's deleted
            
            try:
                os.remove(old_file.path)
            except PermissionError:
                return False
